
import 'dart:async';
import 'dart:convert';

import 'package:dpay_flutter/CheckoutOptions.dart';
import 'package:eventify/eventify.dart';
import 'package:flutter/services.dart';
import 'package:flutter/cupertino.dart';
import 'package:logger/logger.dart';

import 'Constants.dart';

class Durianpay {
  static const MethodChannel _channel =
      const MethodChannel('dpay_flutter');
  var logger = Logger();
  BuildContext globalContext;
// EventEmitter instance used for communication
  EventEmitter _eventEmitter;

  static Durianpay _instance;
  Durianpay._(BuildContext context) {
    _eventEmitter = new EventEmitter();
    globalContext = context;
  }

  static getInstance(BuildContext context) {
    if (Durianpay._instance != null) {
      return Durianpay._instance;
    }

    Durianpay._instance = new Durianpay._(context);

    return Durianpay._instance;
  }

  checkout (DCheckoutOptions options) async{
    logger.d(options.toMap());
    String response = await _channel.invokeMethod("openCheckout", options.toMap());
    logger.d("response in flutter", response);
    handleResponse(response);
  }

  /// Registers event listeners for payment events
  void on (String event, Function handler) {
    EventCallback callBack = (event, context) {
      handler(event.eventData);
    };
    _eventEmitter.on (event, null, callBack);
  }

  /// Clears all event listeners
  void clear() {
    if (_eventEmitter != null) {
      _eventEmitter.clear();
    }
  }

  void handleResponse(String response) {
    var body = json.decode(response);
    if (_eventEmitter != null) {
      switch (body["type"]) {
        case "success" :
          _eventEmitter.emit(PAYMENT_SUCCESS, null, response);
          break;
        case "failure" :
          _eventEmitter.emit(PAYMENT_FAIL, null, response);
          break;
        case "close" :
          _eventEmitter.emit(PAYMENT_CLOSE, null, response);
          break;
      };
    }
  }
}
